/*******************************************************************************
**
**  PROJECT NAME : Voids Bullets 64
**
**  (C)Copyright2005 , Min Chul , Kim ( rageworx@gmail.com )
**
**  All Program codes protects by copyright-law of international.
**
**  Update [2017-10-23] :: mckim - Changed some source codes to modern GCC.
**  Update [2006-03-14] :: mckim - Include "/ds" option for double size window.
**  Update [2005-11-11] :: mckim - include "key up event".
**
********************************************************************************/

#ifdef _WIN32

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <gl/gl.h>
#include "main.h"
#include "sounde.h"
#include "app.h"

/**************************
 * Function Declarations
 **************************/
LRESULT CALLBACK WndProc (HWND hWnd, UINT message,WPARAM wParam, LPARAM lParam);
void EnableOpenGL (HWND hWnd, HDC *hDC, HGLRC *hRC);
void DisableOpenGL (HWND hWnd, HDC hDC, HGLRC hRC);

/**************************
 * Extern from app.c
 **************************/
extern void InitGL(void);
extern void UpdateGL(void);
extern void DrawGL(void);
extern void KeyControl(int key);
extern void KeyUpControl(int key); // 06.01.03 T.W.Kim
extern void CloseGL(void);

/**************************
 * WinMain
 **************************/
int WINAPI WinMain (HINSTANCE hInstance,HINSTANCE hPrevInstance,
                    LPSTR lpCmdLine,int iCmdShow)
{
    WNDCLASS wc;
    HWND hWnd;
    HDC hDC;
    HGLRC hRC;
    MSG msg;
    BOOL bQuit = FALSE;
    BOOL doublesize = FALSE;
    DWORD wsize_w;
    DWORD wsize_h;

	if (!strcmp(lpCmdLine,"/ds")) doublesize = TRUE;

    /* register window class */
    wc.style = CS_OWNDC;
    wc.lpfnWndProc = WndProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon (NULL, IDI_APPLICATION);
    wc.hCursor = LoadCursor (NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH) GetStockObject (BLACK_BRUSH);
    wc.lpszMenuName = NULL;
    wc.lpszClassName = DEF_APP_CLASS_NAME;
    RegisterClass (&wc);

	if (doublesize)
	{
		wsize_w = DEF_APP_FORM_WIDTH * 2;
		wsize_h = DEF_APP_FORM_HEIGHT * 2;
	}
	else
	{
		wsize_w = DEF_APP_FORM_WIDTH;
		wsize_h = DEF_APP_FORM_HEIGHT;
	}

    /* create main window */
    hWnd = CreateWindow ( DEF_APP_CLASS_NAME, DEF_APP_TITLE_NAME,
                          WS_CAPTION |  WS_VISIBLE  ,
                          0, 0, wsize_w , wsize_h ,
                          NULL, NULL, hInstance, NULL);


    /* adjust client rectangle size */
    {
		RECT	surface_rect;
        DWORD   dwStyle;
        int     xp;
        int     yp;
        int     menuFlag;

        /* change window style to popup */
        dwStyle = GetWindowLong(hWnd, GWL_STYLE);

        GetWindowRect(hWnd, &surface_rect);
		xp = 0;
		yp = 0;

        menuFlag    = GetMenu(hWnd) != NULL;
        surface_rect.top      = surface_rect.left = 0;
        surface_rect.right    = wsize_w;
        surface_rect.bottom   = wsize_h;
        AdjustWindowRect(&surface_rect, dwStyle, menuFlag);

        SetWindowPos(hWnd,
					 HWND_NOTOPMOST,
					 xp,
					 yp,
					 surface_rect.right - surface_rect.left,
					 surface_rect.bottom - surface_rect.top,
                     SWP_NOZORDER | SWP_NOACTIVATE);
    }

    /* enable OpenGL for the window */
    EnableOpenGL (hWnd, &hDC, &hRC);

    InitGL();
    //MessageBox(0,"Init_Complete","info",0);

    /* program main loop */
    while (!bQuit)
    {
        /* check for messages */
        if (PeekMessage (&msg, NULL, 0, 0, PM_REMOVE))
        {
            /* handle or dispatch messages */
            if (msg.message == WM_QUIT)
            {
                bQuit = TRUE;
            }
            else
            {
                TranslateMessage (&msg);
                DispatchMessage (&msg);
            }
        }
        else
        {
            UpdateGL();
            DrawGL();
            SwapBuffers (hDC);
            Sleep (1);
        }
    }

    CloseGL();

    /* shutdown OpenGL */
    DisableOpenGL (hWnd, hDC, hRC);

    /* destroy the window explicitly */
    DestroyWindow (hWnd);

    return msg.wParam;
}


/********************
 * Window Procedure
 *
 ********************/

LRESULT CALLBACK WndProc (HWND hWnd, UINT message,
                          WPARAM wParam, LPARAM lParam)
{

    switch (message)
    {
		case WM_CREATE:
			// BASS music play engine _________________________________________
			// Set to 22KHz
			if (!OpenSoundEngine(hWnd,22050))
			{
				MessageBox(0,"DirectX (DirectPlay) can not be initialize !","Error",0);
			}
			return 0;
		case WM_CLOSE:
			CloseSoundEngine();
			PostQuitMessage (0);
			return 0;

		case WM_DESTROY:
			exit(1);
			return 0;

		case WM_KEYDOWN:
			KeyControl(wParam);
			return 0;

		case WM_KEYUP:
			KeyUpControl(wParam);
			return 0;

		default:
			return DefWindowProc (hWnd, message, wParam, lParam);
    }
}


/**************************************************************
 * Enable OpenGL to Window surface
 *************************************************************/
void EnableOpenGL (HWND hWnd, HDC *hDC, HGLRC *hRC)
{
    PIXELFORMATDESCRIPTOR pfd;
    int iFormat;

    /* get the device context (DC) */
    *hDC = GetDC (hWnd);

    /* set the pixel format for the DC */
    ZeroMemory (&pfd, sizeof (pfd));
    pfd.nSize = sizeof (pfd);
    pfd.nVersion = 1;
    pfd.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER;
    pfd.iPixelType = PFD_TYPE_RGBA;
    pfd.cColorBits = DEF_APP_OPENGL_BPP;
    pfd.cDepthBits = DEF_APP_OPENGL_ZBUFFER;
    pfd.iLayerType = PFD_MAIN_PLANE;
    iFormat = ChoosePixelFormat (*hDC, &pfd);
    SetPixelFormat (*hDC, iFormat, &pfd);

    /* create and enable the render context (RC) */
    *hRC = wglCreateContext( *hDC );
    wglMakeCurrent( *hDC, *hRC );

}


/******************
 * Disable OpenGL
 ******************/

void DisableOpenGL (HWND hWnd, HDC hDC, HGLRC hRC)
{
    wglMakeCurrent (NULL, NULL);
    wglDeleteContext (hRC);
    ReleaseDC (hWnd, hDC);
}
#endif // of _WIN32
