/*******************************************************************************
**
**  Sound Device Engine 
**  ===================
**  Win32 uses BASS.DLL (un4seen , copyrighted)
**
*******************************************************************************/
#ifdef _WIN32 /************************[ WINDOWS ]*****************************/
#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "bass.h"

#define DEF_CHANNEL_MAX		4

static unsigned long pchannels[DEF_CHANNEL_MAX] = {0};
static int 			 BASS_opened = 0;

int OpenSoundEngine(HWND win,DWORD freq)
{
	int rtn = 0;
	
	if (BASS_opened) return 0;
	
	rtn = BASS_Init(-1,freq,0,win,NULL);
	if (rtn) BASS_opened = 1;
	
	return rtn;
}

int CloseSoundEngine(void)
{
	if (!BASS_opened) return 0;
	
	BASS_Free();
	
	return 1;
}

int PlayFile(int channel,char *filename,int loop)
{
	int loop_val;
	
	if (loop) 	loop_val = BASS_SAMPLE_LOOP;
	else		loop_val = 0;
	
	if (pchannels[channel])
	{
		BASS_StreamFree(pchannels[channel]);
	}
	
	if (!(pchannels[channel]=BASS_StreamCreateFile(FALSE,filename,0,0,loop_val))
		&& !(pchannels[channel]=BASS_MusicLoad(FALSE,filename,0,0,BASS_MUSIC_RAMP|loop_val,0))) 
	{
		return FALSE; // Can't load the file
	}

	BASS_ChannelPlay(pchannels[channel],FALSE);
}

int StopPlay(int channel)
{
	if (pchannels[channel]) BASS_ChannelStop(pchannels[channel]);
}

#else /****************************[ ANOTHER SYSTEM ]**************************/

int OpenSoundEngine(HWND win,DWORD freq)
{
	return 1;
}

int CloseSoundEngine(void)
{
	return 1;
}

int PlayFile(int channel,char *filename,int loop)
{
	return 1;
}

int StopPlay(int channel)
{
	return 1;
}
#endif /***********************************************************************/
